@@ -34,6 +34,8 @@
 #include "rose_build_castle.h"
 #include "rose_build_convert.h"
 #include "rose_build_dump.h"
+#include "rose_build_groups.h"
+#include "rose_build_matchers.h"
 #include "rose_build_merge.h"
 #include "rose_build_role_aliasing.h"
 #include "rose_build_util.h"
@@ -68,7 +70,6 @@
 #include <algorithm>
 #include <functional>
 #include <map>
-#include <queue>
 #include <set>
 #include <string>
 #include <vector>
@@ -77,65 +78,16 @@
 #include <boost/range/adaptor/map.hpp>
 
 using namespace std;
-using boost::adaptors::map_keys;
 using boost::adaptors::map_values;
 
 namespace ue2 {
 
-#define ROSE_LONG_LITERAL_LEN 8
-
 #define ANCHORED_REHOME_MIN_FLOATING 800
 #define ANCHORED_REHOME_MIN_FLOATING_SHORT 50
 #define ANCHORED_REHOME_ALLOW_SHORT 20
 #define ANCHORED_REHOME_DEEP 25
 #define ANCHORED_REHOME_SHORT_LEN 3
 
-static
-bool superStrong(const rose_literal_id &lit) {
-    if (lit.s.length() < ROSE_LONG_LITERAL_LEN) {
-        return false;
-    }
-
-    const u32 EXPECTED_FDR_BUCKET_LENGTH = 8;
-
-    assert(lit.s.length() >= EXPECTED_FDR_BUCKET_LENGTH);
-    size_t len = lit.s.length();
-    const string &s = lit.s.get_string();
-
-    for (size_t i = 1; i < EXPECTED_FDR_BUCKET_LENGTH; i++) {
-        if (s[len - 1 - i] != s[len - 1]) {
-            return true; /* we have at least some variation in the tail */
-        }
-    }
-    DEBUG_PRINTF("lit '%s' is not superstrong due to tail\n",
-                 escapeString(s).c_str());
-    return false;
-}
-
-rose_group RoseBuildImpl::getGroups(RoseVertex v) const {
-    rose_group groups = 0;
-
-    for (u32 id : g[v].literals) {
-        u32 lit_id = literal_info.at(id).undelayed_id;
-
-        rose_group mygroups = literal_info[lit_id].group_mask;
-        groups |= mygroups;
-    }
-
-    return groups;
-}
-
-/** \brief Get the groups of the successor literals of a given vertex. */
-rose_group RoseBuildImpl::getSuccGroups(RoseVertex start) const {
-    rose_group initialGroups = 0;
-
-    for (auto v : adjacent_vertices_range(start, g)) {
-        initialGroups |= getGroups(v);
-    }
-
-    return initialGroups;
-}
-
 #ifdef DEBUG
 static UNUSED
 void printLitInfo(const rose_literal_info &li, u32 id) {
@@ -481,6 +433,9 @@ RoseRoleHistory findHistoryScheme(const RoseBuildImpl &tbi, const RoseEdge &e) {
 
         // If the bounds are {0,0}, this role can only match precisely at EOD.
         if (minBound == 0 && maxBound == 0) {
+            /* last byte history will squash the state byte so cannot have other
+             * succ */
+            assert(out_degree(u, g) == 1);
             return ROSE_ROLE_HISTORY_LAST_BYTE;
         }
 
@@ -501,7 +456,8 @@ RoseRoleHistory findHistoryScheme(const RoseBuildImpl &tbi, const RoseEdge &e) {
         return ROSE_ROLE_HISTORY_NONE;
     }
 
-    if (g[u].fixedOffset()) {
+    if (g[u].fixedOffset() &&
+        (g[e].minBound || g[e].maxBound != ROSE_BOUND_INF)) {
         DEBUG_PRINTF("fixed offset -> anch\n");
         return ROSE_ROLE_HISTORY_ANCH;
     }
@@ -555,8 +511,8 @@ bool RoseBuildImpl::isDirectReport(u32 id) const {
         }
 
         // Use the program to handle cases that aren't external reports.
-        for (const ReportID &id : g[v].reports) {
-            if (!isExternalReport(rm.getReport(id))) {
+        for (const ReportID &rid : g[v].reports) {
+            if (!isExternalReport(rm.getReport(rid))) {
                 return false;
             }
         }
@@ -585,6 +541,45 @@ bool RoseBuildImpl::isDirectReport(u32 id) const {
     return true;
 }
 
+
+/* If we have prefixes that can squash all the floating roots, we can have a
+ * somewhat-conditional floating table. As we can't yet look at squash_masks, we
+ * have to make some guess as to if we are in this case but the win for not
+ * running a floating table over a large portion of the stream is significantly
+ * larger than avoiding running an eod table over the last N bytes. */
+static
+bool checkFloatingKillableByPrefixes(const RoseBuildImpl &tbi) {
+    for (auto v : vertices_range(tbi.g)) {
+        if (!tbi.isRootSuccessor(v)) {
+            continue;
+        }
+
+        if (!tbi.isFloating(v)) {
+            continue;
+        }
+
+        if (!tbi.g[v].left) {
+            DEBUG_PRINTF("unguarded floating root\n");
+            return false;
+        }
+
+        if (tbi.g[v].left.graph) {
+            const NGHolder &h = *tbi.g[v].left.graph;
+            if (proper_out_degree(h.startDs, h)) {
+                DEBUG_PRINTF("floating nfa prefix, won't die\n");
+                return false;
+            }
+        } else if (tbi.g[v].left.dfa) {
+            if (tbi.g[v].left.dfa->start_floating != DEAD_STATE) {
+                DEBUG_PRINTF("floating dfa prefix, won't die\n");
+                return false;
+            }
+        }
+    }
+
+    return true;
+}
+
 static
 bool checkEodStealFloating(const RoseBuildImpl &tbi,
                            const vector<u32> &eodLiteralsForFloating,
@@ -606,6 +601,11 @@ bool checkEodStealFloating(const RoseBuildImpl &tbi,
         return false;
     }
 
+    if (checkFloatingKillableByPrefixes(tbi)) {
+         DEBUG_PRINTF("skipping as prefixes may make ftable conditional\n");
+         return false;
+    }
+
     DEBUG_PRINTF("%zu are eod literals, %u floating; floating len=%zu\n",
                  eodLiteralsForFloating.size(), numFloatingLiterals,
                  shortestFloatingLen);
@@ -862,274 +862,6 @@ bool RoseBuildImpl::hasFinalId(u32 id) const {
     return literal_info.at(id).final_id != MO_INVALID_IDX;
 }
 
-static
-bool eligibleForAlwaysOnGroup(const RoseBuildImpl &tbi, u32 id) {
-    /* returns true if it or any of its delay versions have root role */
-    for (auto v : tbi.literal_info[id].vertices) {
-        if (tbi.isRootSuccessor(v)) {
-            NGHolder *h = tbi.g[v].left.graph.get();
-            if (!h || proper_out_degree(h->startDs, *h)) {
-                return true;
-            }
-        }
-    }
-
-    for (u32 delayed_id : tbi.literal_info[id].delayed_ids) {
-        for (auto v : tbi.literal_info[delayed_id].vertices) {
-            if (tbi.isRootSuccessor(v)) {
-                NGHolder *h = tbi.g[v].left.graph.get();
-                if (!h || proper_out_degree(h->startDs, *h)) {
-                    return true;
-                }
-            }
-        }
-    }
-
-    return false;
-}
-
-static
-bool requires_group_assignment(const rose_literal_id &lit,
-                               const rose_literal_info &info) {
-    if (lit.delay) { /* we will check the shadow's master */
-        return false;
-    }
-
-    if (lit.table == ROSE_ANCHORED || lit.table == ROSE_EVENT) {
-        return false;
-    }
-
-    // If we already have a group applied, skip.
-    if (info.group_mask) {
-        return false;
-    }
-
-    if (info.vertices.empty() && info.delayed_ids.empty()) {
-        DEBUG_PRINTF("literal is good for nothing\n");
-        return false;
-    }
-
-    return true;
-}
-
-static
-rose_group calcLocalGroup(const RoseVertex v, const RoseGraph &g,
-                          const deque<rose_literal_info> &literal_info,
-                          const bool small_literal_count) {
-    rose_group local_group = 0;
-
-    for (auto u : inv_adjacent_vertices_range(v, g)) {
-        /* In small cases, ensure that siblings have the same rose parentage to
-         * allow rose squashing. In larger cases, don't do this as groups are
-         * probably too scarce. */
-        for (auto w : adjacent_vertices_range(u, g)) {
-            if (!small_literal_count || g[v].left == g[w].left) {
-                for (u32 lit_id : g[w].literals) {
-                    local_group |= literal_info[lit_id].group_mask;
-                }
-            } else {
-                DEBUG_PRINTF("not sibling different mother %zu %zu\n",
-                             g[v].idx, g[w].idx);
-            }
-        }
-    }
-
-    return local_group;
-}
-
-/* group constants */
-#define MAX_LIGHT_LITERAL_CASE 200 /* allow rose to affect group decisions below
-                                    * this */
-
-static
-flat_set<RoseVertex> getAssociatedVertices(const RoseBuildImpl &build, u32 id) {
-    flat_set<RoseVertex> out;
-    const auto &info = build.literal_info[id];
-    insert(&out, info.vertices);
-    for (const auto &delayed : info.delayed_ids) {
-        insert(&out, build.literal_info[delayed].vertices);
-    }
-    return out;
-}
-
-static
-u32 next_available_group(u32 counter, u32 min_start_group) {
-    counter++;
-    if (counter == ROSE_GROUPS_MAX) {
-        DEBUG_PRINTF("resetting groups\n");
-        counter = min_start_group;
-    }
-
-    return counter;
-}
-
-// Assigns groups to literals in the general case, when we have more literals
-// than available groups.
-void RoseBuildImpl::assignGroupsToLiterals() {
-    bool small_literal_count = literal_info.size() <= MAX_LIGHT_LITERAL_CASE;
-
-    map<u8, u32> groupCount; /* group index to number of members */
-
-    u32 counter = 0;
-    u32 group_always_on = 0;
-
-    // First pass: handle always on literals.
-    for (const auto &e : literals.right) {
-        u32 id = e.first;
-        const rose_literal_id &lit = e.second;
-        rose_literal_info &info = literal_info[id];
-
-        if (!requires_group_assignment(lit, info)) {
-            continue;
-        }
-
-        // If this literal has a root role, we always have to search for it
-        // anyway, so it goes in the always-on group.
-        /* We could end up squashing it if it is followed by a .* */
-        if (eligibleForAlwaysOnGroup(*this, id)) {
-            info.group_mask = 1ULL << group_always_on;
-            groupCount[group_always_on]++;
-            continue;
-        }
-    }
-
-    u32 group_long_lit;
-    if (groupCount[group_always_on]) {
-        DEBUG_PRINTF("%u always on literals\n", groupCount[group_always_on]);
-        group_long_lit = group_always_on;
-        counter++;
-    } else {
-        group_long_lit = counter;
-        counter++;
-    }
-
-    u32 min_start_group = counter;
-    priority_queue<pair<pair<s32, s32>, u32> > pq;
-
-    // Second pass: the other literals.
-    for (const auto &e : literals.right) {
-        u32 id = e.first;
-        const rose_literal_id &lit = e.second;
-        rose_literal_info &info = literal_info[id];
-
-        if (!requires_group_assignment(lit, info)) {
-            continue;
-        }
-
-        assert(!eligibleForAlwaysOnGroup(*this, id));
-        pq.push(make_pair(make_pair(-(s32)literal_info[id].vertices.size(),
-                                    -(s32)lit.s.length()), id));
-    }
-
-    vector<u32> long_lits;
-    while (!pq.empty()) {
-        u32 id = pq.top().second;
-        pq.pop();
-        UNUSED const rose_literal_id &lit = literals.right.at(id);
-        DEBUG_PRINTF("assigning groups to lit %u (v %zu l %zu)\n", id,
-                     literal_info[id].vertices.size(), lit.s.length());
-
-        u8 group_id = 0;
-        rose_group group = ~0ULL;
-        for (auto v : getAssociatedVertices(*this, id)) {
-            rose_group local_group = calcLocalGroup(v, g, literal_info,
-                                                    small_literal_count);
-            group &= local_group;
-            if (!group) {
-                break;
-            }
-        }
-
-        if (group == ~0ULL) {
-            goto boring;
-        }
-
-        group &= ~((1ULL << min_start_group) - 1); /* ensure the purity of the
-                                                    * always_on groups */
-        if (!group) {
-            goto boring;
-        }
-
-        group_id = ctz64(group);
-
-        /* TODO: fairness */
-        DEBUG_PRINTF("picking sibling group %hhd\n", group_id);
-        literal_info[id].group_mask = 1ULL << group_id;
-        groupCount[group_id]++;
-
-        continue;
-
-    boring:
-        /* long literals will either be stuck in a mega group or spread around
-         * depending on availability */
-        if (superStrong(lit)) {
-            long_lits.push_back(id);
-            continue;
-        }
-
-        // Other literals are assigned to our remaining groups round-robin.
-        group_id = counter;
-
-        DEBUG_PRINTF("picking boring group %hhd\n", group_id);
-        literal_info[id].group_mask = 1ULL << group_id;
-        groupCount[group_id]++;
-        counter = next_available_group(counter, min_start_group);
-    }
-
-    /* spread long literals out amongst unused groups if any, otherwise stick
-     * them in the always on the group */
-
-    if (groupCount[counter]) {
-        DEBUG_PRINTF("sticking long literals in the image of the always on\n");
-        for (u32 lit_id : long_lits) {
-            literal_info[lit_id].group_mask = 1ULL << group_long_lit;
-            groupCount[group_long_lit]++;
-        }
-    } else {
-        u32 min_long_counter = counter;
-        DEBUG_PRINTF("base long lit group = %u\n", min_long_counter);
-        for (u32 lit_id : long_lits) {
-            u8 group_id = counter;
-            literal_info[lit_id].group_mask = 1ULL << group_id;
-            groupCount[group_id]++;
-            counter = next_available_group(counter, min_long_counter);
-        }
-    }
-
-    /* assign delayed literals to the same group as their parent */
-    for (const auto &e : literals.right) {
-        u32 id = e.first;
-        const rose_literal_id &lit = e.second;
-
-        if (!lit.delay) {
-            continue;
-        }
-
-        u32 parent = literal_info[id].undelayed_id;
-        DEBUG_PRINTF("%u is shadow picking up groups from %u\n", id, parent);
-        assert(literal_info[parent].undelayed_id == parent);
-        assert(literal_info[parent].group_mask);
-        literal_info[id].group_mask = literal_info[parent].group_mask;
-        /* don't increment the group count - these don't really exist */
-    }
-
-    DEBUG_PRINTF("populate group to literal mapping\n");
-    for (const u32 id : literals.right | map_keys) {
-        rose_group groups = literal_info[id].group_mask;
-        while (groups) {
-            u32 group_id = findAndClearLSB_64(&groups);
-            group_to_literal[group_id].insert(id);
-        }
-    }
-
-    /* find how many groups we allocated */
-    for (u32 i = 0; i < ROSE_GROUPS_MAX; i++) {
-        if (groupCount[i]) {
-            group_end = MAX(group_end, i + 1);
-        }
-    }
-}
-
 bool RoseBuildImpl::hasDelayedLiteral(RoseVertex v) const {
     for (u32 lit_id : g[v].literals) {
         if (literals.right.at(lit_id).delay) {
@@ -1160,213 +892,6 @@ bool RoseBuildImpl::hasAnchoredTablePred(RoseVertex v) const {
     return false;
 }
 
-/* returns true if every vertex associated with a groups also belongs to
-   lit_info */
-static
-bool coversGroup(const RoseBuildImpl &tbi, const rose_literal_info &lit_info) {
-    if (lit_info.vertices.empty()) {
-        DEBUG_PRINTF("no vertices - does not cover\n");
-        return false;
-    }
-
-    if (!lit_info.group_mask) {
-        DEBUG_PRINTF("no group - does not cover\n");
-        return false; /* no group (not a floating lit?) */
-    }
-
-    assert(popcount64(lit_info.group_mask) == 1);
-
-    /* for each lit in group, ensure that vertices are a subset of lit_info's */
-    rose_group groups = lit_info.group_mask;
-    while (groups) {
-        u32 group_id = findAndClearLSB_64(&groups);
-        for (u32 id : tbi.group_to_literal.at(group_id)) {
-            DEBUG_PRINTF(" checking against friend %u\n", id);
-            if (!is_subset_of(tbi.literal_info[id].vertices,
-                              lit_info.vertices)) {
-                DEBUG_PRINTF("fail\n");
-                return false;
-            }
-        }
-    }
-
-    DEBUG_PRINTF("ok\n");
-    return true;
-}
-
-static
-bool isGroupSquasher(const RoseBuildImpl &tbi, const u32 id /* literal id */,
-                     rose_group forbidden_squash_group) {
-    const RoseGraph &g = tbi.g;
-
-    const rose_literal_info &lit_info = tbi.literal_info.at(id);
-
-    DEBUG_PRINTF("checking if %u '%s' is a group squasher %016llx\n", id,
-                  dumpString(tbi.literals.right.at(id).s).c_str(),
-                  lit_info.group_mask);
-
-    if (tbi.literals.right.at(id).table == ROSE_EVENT) {
-        DEBUG_PRINTF("event literal, has no groups to squash\n");
-        return false;
-    }
-
-    if (!coversGroup(tbi, lit_info)) {
-        DEBUG_PRINTF("does not cover group\n");
-        return false;
-    }
-
-    if (lit_info.group_mask & forbidden_squash_group) {
-        /* probably a delayed lit */
-        DEBUG_PRINTF("skipping as involves a forbidden group\n");
-        return false;
-    }
-
-    // Single-vertex, less constrained case than the multiple-vertex one below.
-    if (lit_info.vertices.size() == 1) {
-        const RoseVertex &v = *lit_info.vertices.begin();
-
-        if (tbi.hasDelayPred(v)) { /* due to rebuild issues */
-            return false;
-        }
-
-        /* there are two ways to be a group squasher:
-         * 1) only care about the first accepted match
-         * 2) can only match once after a pred match
-         *
-         * (2) requires analysis of the infix before v and is not implemented,
-         * TODO
-         */
-
-        /* Case 1 */
-
-        // Can't squash cases with accepts
-        if (!g[v].reports.empty()) {
-            return false;
-        }
-
-        /* Can't squash cases with a suffix without analysis of the suffix.
-         * TODO: look at suffixes */
-        if (g[v].suffix) {
-            return false;
-        }
-
-        // Out-edges must have inf max bound, + no other shenanigans */
-        for (const auto &e : out_edges_range(v, g)) {
-            if (g[e].maxBound != ROSE_BOUND_INF) {
-                return false;
-            }
-
-            if (g[target(e, g)].left) {
-                return false; /* is an infix rose trigger, TODO: analysis */
-            }
-        }
-
-        DEBUG_PRINTF("%u is a path 1 group squasher\n", id);
-        return true;
-
-        /* note: we could also squash the groups of its preds (if nobody else is
-         * using them. TODO. */
-    }
-
-    // Multiple-vertex case
-    for (auto v : lit_info.vertices) {
-        assert(!tbi.isAnyStart(v));
-
-        // Can't squash cases with accepts
-        if (!g[v].reports.empty()) {
-            return false;
-        }
-
-        // Suffixes and leftfixes are out too as first literal may not match
-        // for everyone.
-        if (!g[v].isBoring()) {
-            return false;
-        }
-
-        /* TODO: checks are solid but we should explain */
-        if (tbi.hasDelayPred(v) || tbi.hasAnchoredTablePred(v)) {
-            return false;
-        }
-
-        // Out-edges must have inf max bound and not directly lead to another
-        // vertex with this group, e.g. 'foobar.*foobar'.
-        for (const auto &e : out_edges_range(v, g)) {
-            if (g[e].maxBound != ROSE_BOUND_INF) {
-                return false;
-            }
-            RoseVertex t = target(e, g);
-
-            if (g[t].left) {
-                return false; /* is an infix rose trigger */
-            }
-
-            for (u32 lit_id : g[t].literals) {
-                if (tbi.literal_info[lit_id].group_mask & lit_info.group_mask) {
-                    return false;
-                }
-            }
-        }
-
-        // In-edges must all be dot-stars with no overlap at all, as overlap
-        // also causes history to be used.
-        /* Different tables are already forbidden by previous checks */
-        for (const auto &e : in_edges_range(v, g)) {
-            if (!(g[e].minBound == 0 && g[e].maxBound == ROSE_BOUND_INF)) {
-                return false;
-            }
-
-            // Check overlap, if source was a literal.
-            RoseVertex u = source(e, g);
-            if (tbi.maxLiteralOverlap(u, v)) {
-                return false;
-            }
-        }
-    }
-
-    DEBUG_PRINTF("literal %u is a multi-vertex group squasher\n", id);
-    return true;
-}
-
-static
-void findGroupSquashers(RoseBuildImpl &tbi) {
-    rose_group forbidden_squash_group = 0;
-    for (const auto &e : tbi.literals.right) {
-        if (e.second.delay) {
-            forbidden_squash_group |= tbi.literal_info[e.first].group_mask;
-        }
-    }
-
-    for (u32 id = 0; id < tbi.literal_info.size(); id++) {
-        if (isGroupSquasher(tbi, id, forbidden_squash_group)) {
-            tbi.literal_info[id].squash_group = true;
-        }
-    }
-}
-
-/**
- * The groups that a role sets are determined by the union of its successor
- * literals. Requires the literals already have had groups assigned.
- */
-void RoseBuildImpl::assignGroupsToRoles() {
-    /* Note: if there is a succ literal in the sidematcher, its successors
-     * literals must be added instead */
-    for (auto v : vertices_range(g)) {
-        if (isAnyStart(v)) {
-            continue;
-        }
-
-        const rose_group succ_groups = getSuccGroups(v);
-        g[v].groups |= succ_groups;
-
-        if (ghost.find(v) != ghost.end()) {
-            /* delayed roles need to supply their groups to the ghost role */
-            g[ghost[v]].groups |= succ_groups;
-        }
-
-        DEBUG_PRINTF("vertex %zu: groups=%llx\n", g[v].idx, g[v].groups);
-    }
-}
-
 void RoseBuildImpl::findTransientLeftfixes(void) {
     for (auto v : vertices_range(g)) {
         if (!g[v].left) {
@@ -1393,19 +918,32 @@ void RoseBuildImpl::findTransientLeftfixes(void) {
             continue;
         }
 
-        u32 his = g[v].left.lag + max_width;
+        if (cc.streaming) {
+            /* STREAMING: transient prefixes must be able to run using history
+             * rather than storing state. */
+            u32 his = g[v].left.lag + max_width;
 
-        // If this vertex has an event literal, we need to add one to cope
-        // with it.
-        if (hasLiteralInTable(v, ROSE_EVENT)) {
-            his++;
-        }
+            // If this vertex has an event literal, we need to add one to cope
+            // with it.
+            if (hasLiteralInTable(v, ROSE_EVENT)) {
+                his++;
+            }
 
-        /* +1 as trigger must appear in main buffer and no byte is needed to
-         * decompress the state */
-        if (his <= cc.grey.maxHistoryAvailable + 1) {
-            transient.insert(left);
-            DEBUG_PRINTF("a transient leftfix has been spotted his=%u\n", his);
+            /* +1 as trigger must appear in main buffer and no byte is needed to
+             * decompress the state */
+            if (his <= cc.grey.maxHistoryAvailable + 1) {
+                transient.insert(left);
+                DEBUG_PRINTF("a transient leftfix spotted his=%u\n", his);
+            }
+        } else {
+            /* BLOCK: transientness is less important and more fuzzy, ideally
+             * it should be quick to calculate the state. No need to worry about
+             * history (and hence lag). */
+            if (max_width < depth(ROSE_BLOCK_TRANSIENT_MAX_WIDTH)) {
+                transient.insert(left);
+                DEBUG_PRINTF("a transient block leftfix spotted [%u]\n",
+                             (u32)max_width);
+            }
         }
     }
 }
@@ -1718,7 +1256,8 @@ void addSmallBlockLiteral(RoseBuildImpl &tbi, const simple_anchored_info &sai,
         assert(old_id < tbi.literal_info.size());
         const rose_literal_info &li = tbi.literal_info[old_id];
 
-        // For compile determinism, operate over literal vertices in index order.
+        // For compile determinism, operate over literal vertices in index
+        // order.
         vector<RoseVertex> lit_verts(begin(li.vertices), end(li.vertices));
         sort(begin(lit_verts), end(lit_verts), VertexIndexComp(g));
 
@@ -1732,40 +1271,9 @@ void addSmallBlockLiteral(RoseBuildImpl &tbi, const simple_anchored_info &sai,
             g[v].max_offset = sai.max_bound + sai.literal.length();
             lit_info.vertices.insert(v);
 
-            assert(!g[v].reports.empty());
-
-            bool doDirectReports = true;
-            for (ReportID report_id : g[v].reports) {
-                const Report &old_rep = tbi.rm.getReport(report_id);
-                if (!isExternalReport(old_rep) || old_rep.hasBounds()) {
-                    doDirectReports = false;
-                    break;
-                }
-            }
-
-            if (doDirectReports) {
-                flat_set<ReportID> dr_reports;
-                for (ReportID report_id : g[v].reports) {
-                    // These new literal roles can be made direct reports, with
-                    // their bounds handled by the bounds on their Report
-                    // structures.
-                    Report rep(tbi.rm.getReport(report_id)); // copy
-                    assert(!rep.hasBounds());
-                    rep.minOffset = sai.literal.length() + sai.min_bound;
-                    rep.maxOffset = sai.literal.length() + sai.max_bound;
-                    dr_reports.insert(tbi.rm.getInternalId(rep));
-                }
-                g[v].reports = dr_reports;
-                RoseEdge e = add_edge(tbi.root, v, g).first;
-                g[e].minBound = 0;             // handled by internal_report
-                g[e].maxBound = ROSE_BOUND_INF; // handled by internal_report
-            } else {
-                // If we have a complex internal report, these must become
-                // anchored literals with their own roles.
-                RoseEdge e = add_edge(anchored_root, v, g).first;
-                g[e].minBound = sai.min_bound;
-                g[e].maxBound = sai.max_bound;
-            }
+            RoseEdge e = add_edge(anchored_root, v, g).first;
+            g[e].minBound = sai.min_bound;
+            g[e].maxBound = sai.max_bound;
         }
     }
 }
@@ -2181,8 +1689,10 @@ aligned_unique_ptr<RoseEngine> RoseBuildImpl::buildRose(u32 minWidth) {
 
     assert(!danglingVertexRef(*this));
 
-    assignGroupsToLiterals();
-    assignGroupsToRoles();
+    findMoreLiteralMasks(*this);
+
+    assignGroupsToLiterals(*this);
+    assignGroupsToRoles(*this);
     findGroupSquashers(*this);
 
     /* final prep work */
