@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, Intel Corporation
+ * Copyright (c) 2015-2016, Intel Corporation
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -158,7 +158,7 @@ void findPaths(const NGHolder &g, CorpusProperties &cProps,
         DEBUG_PRINTF("dequeuing path %s, back %u\n",
                      pathToString(g, *p).c_str(), g[u].index);
 
-        NFAGraph::adjacency_iterator ai, ae;
+        NGHolder::adjacency_iterator ai, ae;
         for (tie(ai, ae) = adjacent_vertices(u, g); ai != ae; ++ai) {
             NFAVertex v = *ai;
 
